# -*- coding:utf-8 -*-
import logging
import contextlib
from io import StringIO
from prestring.output import SeparatedOutput  # NOQA
from prestring.utils import (  # NOQA
    reify,
    LazyFormat,
    LazyArguments,
    LazyJoin,
    LazyKeywords,
    LazyKeywordsRepr,
    LazyArgumentsAndKeywords,
    NameStore,
)
logger = logging.getLogger(__name__)


class Newline(object):
    pass


NEWLINE = Newline()

INDENT = object()
UNINDENT = object()


class PreString(object):
    def __init__(self, value, other=None):
        self.body = [value]
        if other is not None:
            self.body.append(other)

    def clear(self):
        self.body.clear()

    def __iadd__(self, value):
        self.body.append(value)
        return self

    def __add__(self, value):
        return self.__class__(self, value)

    def __iter__(self):
        for v in self.body:
            if isinstance(v, PreString):
                yield from iter(v)
            else:
                yield v

    def __str__(self):
        return "".join(str(v) for v in self)

    def insert_before(self, value):
        self.body.insert(0, value)

    def insert_after(self, value):
        if isinstance(self.body[-1], Newline):
            self.body.pop()
            self.append(value)
            self.body.append(NEWLINE)
        else:
            self.append(value)

    def append(self, value):
        self.body.append(value)

    def tail(self):
        return self.body[-1]

    def pop(self):
        self.body.pop()

    def head(self):
        return self.body[0]


class Sentence(object):
    def __init__(self):
        self.body = []
        self.newline = None

    def eat(self, v):
        self.body.append(v)
        return self

    def is_empty(self):
        return all(x == "" for x in self.body)

    def __str__(self):
        return "".join(map(str, self.body))


class MultiSentence(object):
    def __init__(self, *lines):
        self.lines = lines

    def iterator(self, sentence):
        if not sentence.is_empty():
            yield NEWLINE
        for line in self.lines:
            yield line
            yield NEWLINE

    def as_token(self, lexer, tokens, sentence):
        lexer.loop(tokens, sentence, self.iterator(sentence))
        return Sentence()


class Lexer(object):
    def __init__(self, container_factory, sentence_factory):
        self.container_factory = container_factory or list
        self.sentence_factory = sentence_factory or Sentence

    def loop(self, tokens, sentence, iterator):
        for v in iterator:
            if isinstance(v, Newline):
                sentence.newline = v
                tokens.append(sentence)
                sentence = self.sentence_factory()
            elif hasattr(v, "as_token"):
                sentence = v.as_token(self, tokens, sentence)
            elif v is INDENT or v is UNINDENT:
                tokens.append(v)
            else:
                sentence.eat(v)
        return (tokens, sentence)

    def __call__(self, prestring):
        tokens = self.container_factory()
        sentence = self.sentence_factory()

        tokens, sentence = self.loop(tokens, sentence, prestring)
        if not sentence.is_empty():
            tokens.append(sentence)
        return tokens


class FrameList(object):
    def __init__(self):
        self.framelist = [[]]
        self.level = 0
        self.current = self.framelist[-1]

    def __getitem__(self, k):
        return self.framelist[k]

    def push_frame(self):
        self.current = []
        self.level += 1
        target = self.framelist
        for i in range(self.level):
            target = target[-1]
        target.append(self.current)
        return self.current

    def pop_frame(self):
        current = []
        self.level -= 1
        target = self.framelist
        for i in range(self.level):
            target = target[-1]
        if not target[-1]:
            target.append(current)
        self.current = target[-1]
        return self.current


class Parser(object):
    def __init__(self, framelist_factory):
        self.framelist_factory = framelist_factory

    def __call__(self, tokens):
        framelist = self.framelist_factory()
        for v in tokens:
            if v is INDENT:
                framelist.push_frame()
            elif v is UNINDENT:
                framelist.pop_frame()
            else:
                framelist.current.append(v)
            # todo: command?
        return framelist


class Application(object):
    def __call__(self, framelist, evaluator):
        for frame in framelist[:-1]:
            evaluator.evaluate(frame)
            evaluator.evaluate_newframe()
        evaluator.evaluate(framelist[-1])
        return evaluator


class Evaluator(object):
    def __init__(self, io, indent="    ", newline="\n"):
        self.io = io
        self.indent = indent
        self.newline = newline

    def evaluate(self, frame, i=0):
        if not frame:
            return
        for code in frame[:-1]:
            self._evaluate(code, i)
            self.evaluate_newline(code, i)
        self._evaluate(frame[-1], i)

    def _evaluate(self, code, i):
        if isinstance(code, (list, tuple)):
            self.evaluate(code, i + 1)
        else:
            sentence = str(code)
            if sentence == "":
                return
            self.evaluate_indent(i)
            self.io.write(sentence)  # Sentence is also ok.

    def evaluate_newline(self, code, i):
        self.io.write(self.newline)

    def evaluate_newframe(self):
        self.io.write(self.newline)

    def evaluate_indent(self, i):
        for _ in range(i):
            self.io.write(self.indent)

    def __str__(self):
        return self.io.getvalue().rstrip()


class Module(object):
    def create_body(self, value, other=None):
        return PreString(value)

    def create_evaulator(self):
        return Evaluator(StringIO(), newline=self.newline, indent=self.indent)

    def __init__(
        self, value="", newline="\n", indent="    ", lexer=None, parser=None, application=None
    ):
        self.body = self.create_body(value)
        self.indent = indent
        self.newline = newline
        self.lexer = lexer or Lexer(container_factory=list, sentence_factory=Sentence)
        self.parser = parser or Parser(framelist_factory=FrameList)
        self.application = application or Application()

    def clear(self):
        self.body.clear()

    def unnewline(self):
        if self.body.tail() == NEWLINE:
            self.body.pop()

    def submodule(self, value="", newline=True, factory=None):
        factory = factory or self.__class__
        submodule = factory(
            indent=self.indent,
            newline=self.newline,
            lexer=self.lexer,
            parser=self.parser,
            application=self.application
        )
        if value == "" or not newline:
            submodule.append(value)
        else:
            submodule.stmt(value)
        self.body.append(submodule.body)
        return submodule

    def stmt(self, fmt, *args, **kwargs):
        if args or kwargs:
            value = self.format(fmt, *args, **kwargs)
        else:
            value = fmt
        self.body.append(value)
        self.body.append(NEWLINE)
        return self

    @contextlib.contextmanager
    def scope(self):
        try:
            self.body.append(INDENT)
            yield
        finally:
            self.body.append(UNINDENT)

    def insert_before(self, value):
        self.body.insert_before(value)

    def append(self, value):
        self.body.append(value)

    def insert_after(self, value):
        self.body.insert_after(value)

    def __str__(self):
        evaluator = self.create_evaulator()
        tokens = self.lexer(self.body)
        framelist = self.parser(tokens)
        return str(self.application(framelist, evaluator))

    format = LazyFormat



# -*- coding:utf-8 -*-
import logging
import os.path
import glob
from collections import namedtuple, defaultdict


logger = logging.getLogger(__name__)
File = namedtuple("File", "name, m")


class SeparatedOutput(object):
    @staticmethod
    def module_factory():
        raise NotImplementedError("e.g. python.PythonModule()")

    def __init__(self, dirname, prefix="autogen_", module_factory=None):
        self.dirname = dirname
        self.prefix = prefix
        self.arrived = set()
        self.files = defaultdict(self.new_file)
        self.module_factory = module_factory or self.__class__.module_factory

    def new_file(self, file_name, m=None):
        dirname, basename = os.path.split(file_name)
        fname = "{}{}".format(self.prefix, basename)
        m = m or self.module_factory()
        return File(name=os.path.join(dirname, fname), m=m)

    def prepare(self, f):
        dirname = os.path.dirname(os.path.join(self.dirname, f.name))
        if dirname in self.arrived:
            return
        self.arrived.add(dirname)
        logger.info("touch directory path=%s", dirname)
        os.makedirs(dirname, exist_ok=True)
        if self.prefix:
            for f in glob.glob(os.path.join(dirname, "{}*.py".format(self.prefix))):
                os.remove(f)

    def output(self):
        for file in self.files.values():
            self.output_file(file)

    def output_file(self, file):
        self.prepare(file)

        path = os.path.join(self.dirname, file.name)
        logger.info("touch file path=%s", path)
        with open(path, "w") as wf:
            wf.write(str(file.m))



from collections import defaultdict


class reify(object):
    def __init__(self, wrapped):
        self.wrapped = wrapped
        try:
            self.__doc__ = wrapped.__doc__
        except:
            pass

    def __get__(self, inst, objtype=None):
        if inst is None:
            return self
        val = self.wrapped(inst)
        setattr(inst, self.wrapped.__name__, val)
        return val


class Caller(object):
    def __init__(self, name):
        self.name = name
        self.kwargs = LazyKeywords([])


class LazyArgumentsAndKeywords(object):
    def __init__(self, args=None, kwargs=None):
        self.args = LazyArguments(args or [])
        self.kwargs = LazyKeywords(kwargs or {})

    def _string(self):
        r = []
        if len(self.args.args):
            r.append(self.args)
        if len(self.kwargs.kwargs):
            r.append(self.kwargs)
        return ", ".join(map(str, r))

    @reify
    def value(self):
        return self._string()

    def __str__(self):
        return self.value


class LazyArguments(object):
    def __init__(self, args=None):
        self.args = args or []

    def _string(self):
        return ", ".join(map(str, self.args))

    @reify
    def value(self):
        return self._string()

    def __str__(self):
        return self.value


class LazyKeywords(object):
    def __init__(self, kwargs=None):
        self.kwargs = kwargs or {}

    def _string(self):
        return ", ".join(["{}={}".format(str(k), str(v)) for k, v in sorted(self.kwargs.items())])

    @reify
    def value(self):
        return self._string()

    def __str__(self):
        return self.value


class LazyKeywordsRepr(LazyKeywords):
    def _string(self):
        return ", ".join(["{}={}".format(str(k), repr(v)) for k, v in sorted(self.kwargs.items())])


class LazyJoin(object):
    def __init__(self, sep, args):
        self.sep = sep
        self.args = args

    def _string(self):
        return self.sep.join(map(str, self.args))

    @reify
    def value(self):
        return self._string()

    def __str__(self):
        return self.value


class LazyFormat(object):
    def __init__(self, fmt, *args, **kwargs):
        self.fmt = fmt
        self.args = args
        self.kwargs = kwargs

    def _string(self):
        args = map(str, self.args)
        kwargs = {k: str(v) for k, v in self.kwargs.items()}
        return self.fmt.format(*args, **kwargs)

    @reify
    def value(self):
        return self._string()

    def __str__(self):
        return self.value


class NameStore(object):
    def __init__(self):
        self.c = defaultdict(int)
        self.value_map = {}  # (src_type, dst_type) => (name, i)

    def __contains__(self, value):
        return value in self.value_map

    def __setitem__(self, value, name):
        if value not in self.value_map:
            self.value_map[value] = self.get_name(value, name)
            self.c[name] += 1

    def __getitem__(self, value):
        return self.value_map[value]

    def get_name(self, value, name):
        try:
            return self[value]
        except KeyError:
            i = self.c[name]
            return self.new_name(name, i)

    def new_name(self, name, i):
        if i == 0:
            return name
        else:
            return "{}Dup{}".format(name, i)



# -*- coding:utf-8 -*-
import contextlib
import logging
import re
from . import Module as _Module
from . import (
    INDENT,
    UNINDENT,
    NEWLINE,
    LazyFormat,
    LazyArguments,
    LazyJoin
)
logger = logging.getLogger(__name__)


class GoModule(_Module):
    def __init__(self, *args, **kwargs):
        kwargs["indent"] = "\t"
        super(GoModule, self).__init__(*args, **kwargs)

    def sep(self):
        self.body.append(NEWLINE)

    @contextlib.contextmanager
    def block(self, value=None):
        if value is None:
            self.stmt("{")
        else:
            self.body.append(value)
            self.body.append(" {")
            self.body.append(NEWLINE)
        with self.scope():
            yield
        self.stmt("}")

    def comment(self, comment):
        self.stmt(LazyFormat("// {}", comment))

    def package(self, name):
        self.stmt("package {}".format(name))
        self.sep()

    def return_(self, name):
        self.stmt(LazyFormat("return {}", name))

    def type_alias(self, name, value):
        self.stmt(LazyFormat("type {} {}", name, value))
        self.sep()

    @contextlib.contextmanager
    def type_(self, name, *args):
        with self.block(LazyFormat("type {} {}", name, LazyJoin(" ", args))):
            yield
        self.sep()

    @contextlib.contextmanager
    def func(self, name, *args, return_=""):
        with self.block(LazyFormat("func {}({}) {}", name, LazyArguments(args), return_)):
            yield
        self.sep()

    @contextlib.contextmanager
    def method(self, ob, name, *args, return_=""):
        with self.block(LazyFormat("func ({}) {}({}) {}", ob, name, LazyArguments(args), return_)):
            yield
        self.sep()

    @contextlib.contextmanager
    def if_(self, cond):
        with self.block(LazyFormat("if {} ", cond)):
            yield

    @contextlib.contextmanager
    def elif_(self, cond):
        self.unnewline()
        with self.block(LazyFormat(" else if {} ", cond)):
            yield

    @contextlib.contextmanager
    def for_(self, cond):
        with self.block(LazyFormat("for {} ", cond)):
            yield

    @contextlib.contextmanager
    def else_(self):
        self.unnewline()
        with self.block(" else "):
            yield

    def select(self):
        m = self.submodule('select', newline=False)
        return MultiBranchClause(m)

    def switch(self, cond):
        m = self.submodule(LazyFormat('switch {}', cond), newline=False)
        return MultiBranchClause(m)

    def import_group(self):
        m = self.submodule("import", newline=False)
        return ImportGroup(m)

    def const_group(self):
        m = self.submodule("const", newline=False)
        return Group(m)


class MultiBranchClause(object):
    def __init__(self, m):
        self.m = m

    def __getattr__(self, name):
        return getattr(self.m, name)

    def __enter__(self):
        self.m.stmt(' {')
        return self

    @contextlib.contextmanager
    def case(self, value):
        self.m.stmt('case {}:'.format(value))
        with self.m.scope():
            yield

    def __exit__(self, *args, **kwargs):
        self.m.stmt('}')
        self.m.sep()

    @contextlib.contextmanager
    def default(self):
        self.m.stmt('default:')
        with self.m.scope():
            yield


class Group(object):
    def __init__(self, m):
        self.m = m
        self.submodule = None
        self.added = set()

    def __getattr__(self, name):
        return getattr(self.m, name)

    def __enter__(self):
        self.m.stmt(' (')
        self.m.body.append(INDENT)
        self.submodule = self.m.submodule("", newline=False)
        return self

    def __call__(self, name):
        if name in self.added:
            return
        self.added.add(name)
        self.submodule.stmt(name)

    def clear_ifempty(self):
        if str(self.submodule) == "":
            self.m.clear()

    def __exit__(self, *args, **kwargs):
        self.m.body.append(UNINDENT)
        self.m.stmt(')')
        self.m.sep()


class ImportGroup(Group):
    def import_(self, name, as_=None):
        pair = (name, as_)
        if pair in self.added:
            return
        self.added.add(pair)
        if as_ is None:
            self.submodule.stmt('"{}"'.format(name))
        else:
            self.submodule.stmt('{} "{}"'.format(as_, name))
    __call__ = import_

Module = GoModule


# comfortable name expression for go.
def titlize(s):
    if not s:
        return s
    return s[0].upper() + s[1:]


class NameFormatter:
    def format(self, s,
               num_rx=re.compile("\d{2,}"),
               exclude_rx=re.compile("[^a-z0-9]", re.IGNORECASE | re.MULTILINE)):
        if not s:
            return ""
        elif num_rx.match(s):
            return self.format("Num" + s)
        elif s[0] in self.NUMBERS:
            return self.format(self.NUMBERS[s[0]] + s[1:])
        else:
            return exclude_rx.sub("", self.proper_acronym(s))

    def proper_acronym(self, s,
                       rx=re.compile("(?P<sep>^|[^a-zA-Z])(?P<frag>[a-z]+)", re.M),
                       rx2=re.compile("(?P<sep>[A-Z])(?P<frag>[a-z]+)", re.M)):
        return rx2.sub(self._proper_repl2, rx.sub(self._proper_repl1, s))

    NUMBERS = {
        '0': "Zero_", '1': "One_", '2': "Two_", '3': "Three_",
        '4': "Four_", '5': "Five_", '6': "Six_", '7': "Seven_",
        '8': "Eight_", '9': "Nine_"
    }

    # https://github.com/golang/lint/blob/39d15d55e9777df34cdffde4f406ab27fd2e60c0/lint.go#L695-L731
    COMMON_INITIALISMS = [
        "API", "ASCII", "CPU", "CSS", "DNS", "EOF", "GUID", "HTML", "HTTP",
        "HTTPS", "ID", "IP", "JSON", "LHS", "QPS", "RAM", "RHS", "RPC", "SLA",
        "SMTP", "SSH", "TCP", "TLS", "TTL", "UDP", "UI", "UID", "UUID", "URI",
        "URL", "UTF8", "VM", "XML", "XSRF", "XSS"
    ]

    def _proper_repl1(self, m):
        d = m.groupdict()
        if d["frag"].upper() in self.COMMON_INITIALISMS:
            return d["sep"] + d["frag"].upper()
        else:
            return d["sep"] + titlize(d["frag"])

    def _proper_repl2(self, m):
        d = m.groupdict()
        merged = d["sep"] + d["frag"]
        if merged.upper() in self.COMMON_INITIALISMS:
            return merged.upper()
        else:
            return merged


def goname(s, formatter=NameFormatter()):
    return formatter.format(s)



# -*- coding:utf-8 -*-
import contextlib
from io import StringIO
from . import Module as _Module
from . import (
    Newline,
    NEWLINE,
    INDENT,
    UNINDENT,
    Evaluator,
    Sentence,
    LazyArguments,
    LazyKeywords,
    LazyArgumentsAndKeywords,
    LazyFormat
)
from .compat import PY3


PEPNEWLINE = Newline()


class PythonEvaluator(Evaluator):
    def evaluate_newframe(self):
        self.io.write(self.newline)
        self.io.write(self.newline)

    def evaluate_newline(self, code, i):
        self.io.write(self.newline)
        if i <= 0 and hasattr(code, "newline") and code.newline is PEPNEWLINE:
            self.io.write(self.newline)


class PythonModule(_Module):
    def __init__(self, *args, **kwargs):
        self.width = kwargs.pop("width", 100)
        self.import_unique = kwargs.pop("import_unique", False)
        super(PythonModule, self).__init__(*args, **kwargs)
        self.from_map = {}  # module -> PythonModule
        self.imported_set = set()

    def submodule(self, value="", newline=True):
        submodule = super(PythonModule, self).submodule(value=value, newline=newline)
        submodule.width = self.width
        submodule.import_unique = self.import_unique
        return submodule

    def create_evaulator(self):
        return PythonEvaluator(StringIO(), newline=self.newline, indent=self.indent)

    def sep(self):
        self.body.append(PEPNEWLINE)

    def method(self, name, *args, **kwargs):
        return self.def_(name, "self", *args, **kwargs)

    @contextlib.contextmanager
    def with_(self, expr, as_=None):
        if as_:
            self.stmt("with {} as {}:", expr, as_)
        else:
            self.stmt("with {}:", expr)
        with self.scope():
            yield

    @contextlib.contextmanager
    def def_(self, name, *args, **kwargs):
        self.stmt("def {}({}):", name, LazyArgumentsAndKeywords(args, kwargs))
        with self.scope():
            yield
        self.sep()

    @contextlib.contextmanager
    def if_(self, expr):
        self.stmt("if {}:", expr)
        with self.scope():
            yield

    @contextlib.contextmanager
    def unless(self, expr):
        self.stmt("if not ({}):", expr)
        with self.scope():
            yield

    @contextlib.contextmanager
    def elif_(self, expr):
        self.stmt("elif {}:", expr)
        with self.scope():
            yield

    @contextlib.contextmanager
    def else_(self):
        self.stmt("else:")
        with self.scope():
            yield

    @contextlib.contextmanager
    def for_(self, var, iterator):
        self.stmt("for {var} in {iterator}:", var=var, iterator=iterator)
        with self.scope():
            yield

    @contextlib.contextmanager
    def while_(self, expr):
        self.stmt("while {expr}:", expr=expr)
        with self.scope():
            yield

    @contextlib.contextmanager
    def try_(self):
        self.stmt("try:")
        with self.scope():
            yield

    @contextlib.contextmanager
    def except_(self, expr=None):
        if expr:
            self.stmt("except {expr}:", expr=expr)
        else:
            self.stmt("except:")
        with self.scope():
            yield

    @contextlib.contextmanager
    def finally_(self):
        self.stmt("finally:")
        with self.scope():
            yield

    # class definition
    @contextlib.contextmanager
    def class_(self, name, bases=None, metaclass=None):
        if bases is None:
            bases = "object"
        if not isinstance(bases, (list, tuple)):
            bases = [bases]
        args = ", ".join(str(b) for b in bases)
        if PY3:
            if metaclass is not None:
                args += ", metaclass={}".format(metaclass)
            self.stmt("class {name}({args}):", name=name, args=args)
            with self.scope():
                yield
        else:
            self.stmt("class {name}({args}):", name=name, args=args)
            with self.scope():
                if metaclass is not None:
                    self.stmt("__metaclass__ = {}".format(metaclass))
        self.sep()

    @contextlib.contextmanager
    def main(self):
        with self.if_('__name__ == "__main__"'):
            yield

    # sentence
    def break_(self):
        self.stmt("break")

    def continue_(self):
        self.stmt("continue")

    def return_(self, expr, *args):
        self.stmt("return %s" % (expr,), *args)

    def yield_(self, expr, *args):
        self.stmt("yield %s" % (expr,), *args)

    def raise_(self, expr, *args):
        self.stmt("raise %s" % (expr,), *args)

    def import_(self, modname):
        if modname in self.imported_set:
            return
        self.imported_set.add(modname)
        # todo: considering self.import_unique
        self.stmt("import {}", modname)

    def from_(self, modname, *attrs):
        try:
            from_stmt = self.from_map[modname].body.tail()
            for sym in attrs:
                from_stmt.append(sym)
        except KeyError:
            self.from_map[modname] = self.submodule(FromStatement(modname, attrs, unique=self.import_unique), newline=False)

    @contextlib.contextmanager
    def hugecall(self, name):
        caller = Caller(name)
        yield caller
        self.call(caller.name, *caller.args)

    def call(self, name_, *args):
        oneline = LazyFormat("{}({})", name_, LazyArguments(args))
        if len(str(oneline._string())) <= self.width:
            self.stmt(oneline)
        else:
            self.body.append(MultiSentenceForCall(name_, *args))

    def pass_(self):
        self.stmt("pass")


class Caller(object):
    def __init__(self, name):
        self.name = name
        self.kwargs = LazyKeywords([])

    def add(self, **kwargs):
        for k, v in kwargs.items():
            self.kwargs.kwargs[k] = v


class FromStatement(object):
    def __init__(self, modname, symbols, unique=False):
        self.modname = modname
        self.symbols = list(symbols)
        self.unique = unique

    def append(self, symbol):
        self.symbols.append(symbol)

    def stmt(self, s):
        yield s
        yield NEWLINE

    def iterator_for_one_symbol(self, sentence):
        if not sentence.is_empty():
            yield NEWLINE
        yield from self.stmt("from {} import {}".format(self.modname, self.symbols[0]))

    def iterator_for_many_symbols(self, sentence):
        if not sentence.is_empty():
            yield NEWLINE
        yield from self.stmt("from {} import (".format(self.modname))
        yield INDENT
        if self.unique:
            symbols = tuple(sorted(set(self.symbols)))
        else:
            symbols = self.symbols
        for sym in symbols[:-1]:
            yield from self.stmt("{},".format(sym))
        yield from self.stmt("{}".format(symbols[-1]))
        yield UNINDENT
        yield from self.stmt(")")

    def as_token(self, lexer, tokens, sentence):
        if len(self.symbols) <= 1:
            lexer.loop(tokens, sentence, self.iterator_for_one_symbol(sentence))
        else:
            lexer.loop(tokens, sentence, self.iterator_for_many_symbols(sentence))
        return Sentence()


class MultiSentenceForCall(object):
    def __init__(self, name, *lines):
        self.name = name
        self.lines = lines

    def iterator(self, sentence):
        if not sentence.is_empty():
            yield NEWLINE
        yield LazyFormat("{}(", self.name)
        yield NEWLINE
        yield INDENT
        for line in self.lines[:-1]:
            yield LazyFormat("{},", line)
            yield NEWLINE
        yield LazyFormat("{})", self.lines[-1])
        yield NEWLINE
        yield UNINDENT

    def as_token(self, lexer, tokens, sentence):
        lexer.loop(tokens, sentence, self.iterator(sentence))
        return Sentence()


Module = PythonModule



# -*- coding:utf-8 -*-
import sys
import types

# True if we are running on Python 3.
PY3 = sys.version_info[0] == 3


if PY3: # pragma: no cover
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
    long = int
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str
    long = long

def text_(s, encoding='utf-8', errors='strict'):
    """ If ``s`` is an instance of ``binary_type``, return
    ``s.decode(encoding, errors)``, otherwise return ``s``"""
    if isinstance(s, binary_type):
        return s.decode(encoding, errors)
    return s # pragma: no cover

def bytes_(s, encoding='utf-8', errors='strict'):
    """ If ``s`` is an instance of ``text_type``, return
    ``s.encode(encoding, errors)``, otherwise return ``s``"""
    if isinstance(s, text_type): # pragma: no cover
        return s.encode(encoding, errors)
    return s

if PY3:
    FileNotFoundError = FileNotFoundError
else:
    FileNotFoundError = IOError

try:
    from StringIO import StringIO as NativeIO
except ImportError: # pragma: no cover
    from io import StringIO as NativeIO




# -*- coding:utf-8 -*-

import os
import sys

from setuptools import setup, find_packages
here = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(here, 'README.rst')) as f:
        README = f.read()
    with open(os.path.join(here, 'CHANGES.txt')) as f:
        CHANGES = f.read()
except IOError:
    README = CHANGES = ''

install_requires = []

docs_extras = []

tests_require = ["evilunit"]

testing_extras = tests_require + []

setup(
    name='prestring',
    version='0.4.9.2',
    description='source code generation library (with overuse with-syntax)',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    keywords='prestring, srcgen, python',
    author="podhmo",
    author_email="ababjam61@gmail.com",
    url="https://github.com/podhmo/prestring",
    packages=find_packages(exclude=["prestring.tests"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    extras_require={
        'testing': testing_extras,
        'docs': docs_extras,
    },
    tests_require=tests_require,
    test_suite="prestring.tests",
    entry_points="""
"""
)
